# -*- coding: utf-8 -*-

from argparse import ArgumentTypeError
from os.path import isdir
from os.path import isfile
from os.path import sep

from pdf_play.helpers import utils


class _TargetFileOTO:

    def __call__(self, path):
        if not isfile(path):
            raise ArgumentTypeError(f'File does not exits: {path}')
        if not utils.is_pdf(path):
            msg = f'Given target file does not appear to be a PDF: {path}'
        return path


class _OutputFileOTO:

    def __call__(self, path):
        print(f'P: {path}')
        if not _is_pdf(path):
            name_and_extension = path.split(sep)[-1].split('.')
            if len(name_and_extension) == 2:
                extension = name_and_extension[-1]
                if extension != 'pdf':
                    raise ArgumentTypeError(f'Expected a PDF file, received: {path}')
        return path


class _TargetFileOTM:

    def __call__(self, path):
        if utils.is_pdf(path):
            if not isfile(path):
                raise ArgumentTypeError(f'File does not exist: {path}')
        else:
            if not isdir(path):
                raise ArgumentTypeError(f'Expected a PDF file or a directory that '
                                        f'contains PDF files, received: {path}')
        return path


class _OutputFileOTM:

    def __call__(self, path):
        if isfile(path):
            raise ArgumentTypeError(f'Output location must be a directory, not a file: '
                                    f'{path}')
        name_and_extension = path.split(sep)[-1].split('.')
        if len(name_and_extension) == 2:
            raise ArgumentTypeError(f'Output location must be a directory, not a file: '
                                    f'{path}')
        return path


class _WatermarkMTO:

    def __call__(self, path):
        supported = ['.txt']
        if not isfile(path):
            raise ArgumentTypeError(f'Provide path to a valid file to read watermark '
                                    f'texts, given file does not exist: {path}')
        if not any(map(lambda x: x == path[-len(x):], supported)):
            raise ArgumentTypeError(f'Bad input for watermark file: {path}, '
                                    f'please check supported file types: {supported}')
        return path


target_file_oto = _TargetFileOTO()
output_file_oto = _OutputFileOTO()
target_file_otm = _TargetFileOTM()
output_file_otm = _OutputFileOTM()
watermark_mto = _WatermarkMTO()



def is_pdf(path):
    return path.endswith('.pdf')


def flatten(nested_list):
    values = []
    for i in nested_list:
        if isinstance(i, list):
            for j in i:
                values.append(j)
        else:
            values.append(i)
    return values



#!/usr/bin/local/python3.9
# -*- coding: utf-8 -*-

"""
    Builds CLI options
"""

from argparse import ArgumentParser
from os import makedirs
from os.path import isdir
from os.path import join
from os.path import sep
from pathlib import Path

from pdf_play.helpers import _input_actions as actions
from pdf_play.helpers import _input_types as types

_pdf_play = f'''\n{"-" * 100}\n{"*" * 30}{" " * 16}PDF-Play{" " * 16}{"*" * 30
}\n{"-" * 100}\n'''


def _validate_otm(args):
    if args.output_directory is None:
        args.output_directory = 'watermarked'
        if not isdir(args.output_directory):
            print('User did not provide an output location, a directory called '
                  '"watermarked" will be created to store all the watermarked PDF(s).')
            makedirs(args.output_directory)


def _validate_mto(args):
    _validate_otm(args)


def _validate_oto(args):
    if args.output_file is None:
        # print('--> User did not provide an output file, a watermarked copy of the input '
        #       'file will be created.')
        base = Path(args.target_file).parent.resolve()
        file_name = args.target_file.split(sep)[-1]
        name, extension = file_name.split('.')
        args.output_file = join(base, f'{name}_watermarked.{extension}')


def _update_args(args):
    if args.type == 'oto':
        _validate_oto(args)
    if args.type == 'otm':
        _validate_otm(args)
    if args.type == 'mto':
        _validate_mto(args)
    return args


def _validate_args(parser, args):
    if args.command is None:
        print(_pdf_play)
        parser.print_help()
        exit(0)
    else:
        if args.type is None:
            print('\n--> User must specify the mode of operation when trying to '
                  f'watermark PDF files. Get help: "python -m pdf_play  {args.command} '
                  '-h".''')
            exit(0)


def parse_user_args(command=None):
    __usage = 'usage: pdf_play [-h] {watermark} ...'
    _usage = f'{_pdf_play}\n{__usage}'
    try:
        parser = ArgumentParser(prog='pdf_play',
                                description='A Python utility to watermark PDF '
                                            'documents.',
                                epilog='''--> Run -h/--help against indivdual commands to 
                                get more details. Example: "python -m pdf_play watermark 
                                -h"''')
        commands = parser.add_subparsers(dest='command',
                                         help='''Commands available in PDF-Play, 
                                         currently there's just one :(''')
        watermark = commands.add_parser('watermark', help='Watermarks PDF files.',
                                        description='Apply watermarks to PDF '
                                                    'files.',
                                        epilog='''--> Sample usage: python -m pdf_play 
                                        watermark oto -h''')
        sub_commands = watermark.add_subparsers(dest='type',
                                                help='''Modes of operation for 
                                                     watermarking PDF files.''')
        oto = sub_commands.add_parser('oto',
                                      help='''One-To-One: Apply watermark text to a 
                                           single file.''',
                                      description='''One-To-One: Use this mode when you 
                                      have got a single target file.''',
                                      epilog='''--> Sample Usage: python -m pdf_play 
                                             watermark oto -t this is my watermark 
                                             text -i sample.pdf''')
        oto.add_argument('--text', '-t', type=str, default='PDFPlay',
                         dest='text', required=True, nargs='+',
                         action=actions.watermark_text,
                         help='(**) Text that is to be applied as the watermark.')
        oto.add_argument('--input', '-i', default=None, type=types.target_file_oto,
                         action=actions.target_file_oto, dest='target_file',
                         required=True,
                         help='(**) Path to the PDF file that is to be watermarked.')
        oto.add_argument('--output', '-o', default=None, type=types.output_file_oto,
                         action=actions.output_file_oto, dest='output_file',
                         help='Name of the output file, by default '
                              '"{input-file}_watermarked.pdf" will be generated.')
        oto.add_argument('--font-name', '-fn', default='Helvetica-Bold', type=str,
                         choices=['Helvetica-Bold'], dest='font_name',
                         help='Name of the font that you want to use in the watermark.')
        oto.add_argument('--font-size', '-fs', default='medium', type=str,
                         choices=['small', 'medium', 'large'], dest='font_size',
                         help='Size of the font.')
        oto.add_argument('--text-alignment', '-ta', default='diagonal', type=str,
                         choices=['horizontal', 'diagonal'], dest='text_alignment',
                         help='Alignment of the watermark in the document.')

        otm = sub_commands.add_parser('otm', help='One-To-Many: Apply watermark to '
                                                  'multiple files.',
                                      description='One-To-Many: Use this mode when you '
                                                  'have got multiple target files.',
                                      epilog='''--> Sample Usage: python -m pdf_play
                                             watermark otm -t this is my watermark
                                             text -i sample1.pdf sample2.pdf
                                             MyPC/Downloads/PDFFiles ''')
        otm.add_argument('--text', '-t', type=str, default='PDFPlay', dest='text',
                         required=True, action=actions.watermark_text, nargs='+',
                         help='(**) Text that is to be applied as the watermark.')
        otm.add_argument('--input', '-i', default=None, type=types.target_file_otm,
                         action=actions.target_file_otm, dest='target_files', nargs='+',
                         required=True,
                         help='''(**) Paths to files/directories that contain
                         your target files.''')
        otm.add_argument('--output', '-o', default=None,
                         type=types.output_file_otm,
                         action=actions.output_file_otm, dest='output_directory',
                         help='Directory to save watermarked files.')
        otm.add_argument('--font-name', '-fn', default='Helvetica-Bold', type=str,
                         choices=['Helvetica-Bold'], dest='font_name',
                         help='Name of the font that you want to use in the watermark.')
        otm.add_argument('--font-size', '-fs', default='medium', type=str,
                         choices=['small', 'medium', 'large'], dest='font_size',
                         help='Size of the font.')
        otm.add_argument('--text-alignment', '-ta', default='diagonal', type=str,
                         choices=['horizontal', 'diagonal'], dest='text_alignment',
                         help='Alignment of the watermark in the document.')
        #
        # mto = sub_commands.add_parser('mto', help='Many-To-One: Apply many different '
        #                                           'watermarks to a single file',
        #                               description='''Many-To-One: Use this mode when you
        #                               want to watermark a file multiple times with
        #                               different watermark texts.''',
        #                               epilog='''--> Sample usage: python -m pdf_play
        #                               watermark mto -t watermark text#1, watermark
        #                               text#2, MyPC/SampleWatermarks/sample1.txt -i
        #                               sample.pdf''')
        # mto.add_argument('--text', '-t', default='PDFPlay',
        #                  type=types.watermark_mto,
        #                  action=actions.watermark_mto,
        #                  dest='texts',
        #                  help='''(**) Texts that are to be watermarked or path to a txt
        #                       file that contains such texts.
        #                       (Expected comma separated values)''')
        # mto.add_argument('--input', '-i', default=None, type=types.target_file_oto,
        #                  action=actions.target_file_oto, dest='target_file',
        #                  required=True,
        #                  help='(**) Path to the PDF file that is to be watermarked.')
        # mto.add_argument('--output', '-o', default=None,
        #                  type=types.output_file_otm,
        #                  action=actions.output_file_otm, dest='output_directory',
        #                  help='''Directory to save watermarked files.''')
        # mto.add_argument('--font-name', '-fn', default='Helvetica-Bold', type=str,
        #                  choices=['Helvetica-Bold'], dest='font_name',
        #                  help='Name of the font that you want to use in the watermark.')
        # mto.add_argument('--font-size', '-fs', default='medium', type=str,
        #                  choices=['small', 'medium', 'large'], dest='font_size',
        #                  help='Size of the font.')
        # mto.add_argument('--text-alignment', '-ta', default='diagonal', type=str,
        #                  choices=['horizontal', 'diagonal'], dest='text_alignment',
        #                  help='Alignment of the watermark in the document.')

        args = parser.parse_args() if command is None else parser.parse_args(command)
        _validate_args(parser, args)
        args = _update_args(args)
        return args
    except Exception as e:
        print(f'Error --> Bad user-input: {e}')
        raise e


def parse_watermark_args():
    try:
        parser = ArgumentParser(prog='watermark',
                                description='A Python utility to watermark PDF '
                                            'documents.',
                                epilog='''--> Run -h/--help to get more details. 
                                Example: "watermark -h"''')
        commands = parser.add_subparsers(dest='type',
                                         help='''Modes of operation for 
                                                 watermarking PDF files.''')
        oto = commands.add_parser('oto',
                                  help='''One-To-One: Apply watermark text to a 
                                                   single file.''',
                                  description='''One-To-One: Use this mode when you 
                                              have got a single target file.''',
                                  epilog='''--> Sample Usage: python -m pdf_play 
                                                     watermark oto -t this is my watermark 
                                                     text -i sample.pdf''')
        oto.add_argument('--text', '-t', type=str, default='PDFPlay',
                         dest='text', required=True, nargs='+',
                         action=actions.watermark_text,
                         help='(**) Text that is to be applied as the watermark.')
        oto.add_argument('--input', '-i', default=None, type=types.target_file_oto,
                         action=actions.target_file_oto, dest='target_file',
                         required=True,
                         help='(**) Path to the PDF file that is to be watermarked.')
        oto.add_argument('--output', '-o', default=None, type=types.output_file_oto,
                         action=actions.output_file_oto, dest='output_file',
                         help='Name of the output file, by default '
                              '"{input-file}_watermarked.pdf" will be generated.')
        oto.add_argument('--font-name', '-fn', default='Helvetica-Bold', type=str,
                         choices=['Helvetica-Bold'], dest='font_name',
                         help='Name of the font that you want to use in the watermark.')
        oto.add_argument('--font-size', '-fs', default='medium', type=str,
                         choices=['small', 'medium', 'large'], dest='font_size',
                         help='Size of the font.')
        oto.add_argument('--text-alignment', '-ta', default='diagonal', type=str,
                         choices=['horizontal', 'diagonal'], dest='text_alignment',
                         help='Alignment of the watermark in the document.')

        otm = commands.add_parser('otm',
                                  help='One-To-Many: Apply watermark to multiple files.',
                                  description='One-To-Many: Use this mode when you '
                                              'have got multiple target files.',
                                  epilog='''--> Sample Usage: python -m pdf_play
                                             watermark otm -t this is my watermark
                                             text -i sample1.pdf sample2.pdf
                                             MyPC/Downloads/PDFFiles ''')
        otm.add_argument('--text', '-t', type=str, default='PDFPlay', dest='text',
                         required=True, action=actions.watermark_text, nargs='+',
                         help='(**) Text that is to be applied as the watermark.')
        otm.add_argument('--input', '-i', default=None, type=types.target_file_otm,
                         action=actions.target_file_otm, dest='target_files', nargs='+',
                         required=True,
                         help='''(**) Paths to files/directories that contain
                         your target files.''')
        otm.add_argument('--output', '-o', default=None,
                         type=types.output_file_otm,
                         action=actions.output_file_otm, dest='output_directory',
                         help='Directory to save watermarked files.')
        otm.add_argument('--font-name', '-fn', default='Helvetica-Bold', type=str,
                         choices=['Helvetica-Bold'], dest='font_name',
                         help='Name of the font that you want to use in the watermark.')
        otm.add_argument('--font-size', '-fs', default='medium', type=str,
                         choices=['small', 'medium', 'large'], dest='font_size',
                         help='Size of the font.')
        otm.add_argument('--text-alignment', '-ta', default='diagonal', type=str,
                         choices=['horizontal', 'diagonal'], dest='text_alignment',
                         help='Alignment of the watermark in the document.')

        args = parser.parse_args()
        args.command = 'watermark'
        _validate_args(parser, args)
        args = _update_args(args)
        return args
    except Exception as e:
        print(f'Error --> Bad user-input: {e}')
        raise e


def _test():
    command = ['watermark', '-t', 'test target']
    args = parse_user_args(command)
    print(f'User Args: {args}')


if __name__ == '__main__':
    _test()



from os.path import join
from os.path import sep
from pathlib import Path
from pdf_play import __settings__ as settings
from pdf_play.core import watermark
from pdf_play.helpers import parse_user_args
from pdf_play.helpers import parse_watermark_args


def _watermark_mto(texts=('PDFPlay',), target_file=None, output_directory=None,
                   font_name='Helvetica-Bold',
                   font_size='medium', text_alignment='diagonal'):
    raise NotImplementedError('To be implemented...')


def _watermark_otm(text='PDFPlay', target_files=None, output_directory=None,
                   font_name='Helvetica-Bold',
                   font_size='medium', text_alignment='diagonal'):
    for target_file in target_files:
        name_and_extension = target_file.split(sep)[-1]
        name, extension = name_and_extension.split('.')
        output_file = join(output_directory, f'{name}_watermarked.{extension}')
        watermark(text, target_file, output_file, font_name=font_name,
                  font_size=font_size, text_alignment=text_alignment)


def _watermark_oto(text='PDFPlay', target_file=None, output_file=None,
                   font_name='Helvetica-Bold',
                   font_size='medium', text_alignment='diagonal'):
    watermark(text, target_file, output_file, font_name=font_name, font_size=font_size,
              text_alignment=text_alignment)


def _watermark(args=None):
    if args is None:
        args = parse_watermark_args()
    type_ = args.type
    del args.type
    del args.command
    type_map = dict(oto=_watermark_oto, otm=_watermark_otm)
    type_map[type_](**vars(args))


def main():
    args = parse_user_args()
    command_map = dict(watermark=_watermark)
    command_map[args.command](args)



from argparse import Action
from os import listdir
from os import makedirs
from os.path import isdir
from os.path import join
from os.path import sep
from pathlib import Path

from pdf_play.helpers import utils


class _WatermarkText(Action):

    def __call__(self, parser, namespace, values, option_string=None):
        print('')
        setattr(namespace, self.dest, ' '.join(values))


class _TargetFileOTO(Action):

    def __call__(self, parser, namespace, values, option_string=None):
        setattr(namespace, self.dest, values)


class _OutputFileOTO(Action):

    def __call__(self, parser, namespace, values, option_string=None):
        base = Path(values).parent.resolve()
        if not isdir(base):
            makedirs(base)
        if not values.endswith('.pdf'):
            values += '.pdf'
        setattr(namespace, self.dest, values)


class _TargetFileOTM(Action):

    def __call__(self, parser, namespace, values, option_string=None):
        values_ = []
        for value in values:
            if isdir(value):
                pdfs = list(map(lambda x: join(value, x), filter(utils.is_pdf, listdir(
                    value))))
                values_.extend(pdfs)
            else:
                values_.append(value)
        values = list(set(values_))
        setattr(namespace, self.dest, values)


class _OutputFileOTM(Action):

    def __call__(self, parser, namespace, values, option_string=None):
        base = Path(values).parent.resolve()
        target = values.split(sep)[-1]
        existing_dirs = len(list(filter(lambda x: target in x, listdir(base))))
        if existing_dirs:
            values = join(base, f'{target}({existing_dirs + 1})')
        else:
            values = join(base, target)
        if not isdir(values):
            makedirs(values)
        setattr(namespace, self.dest, values)


class _WatermarkMTO(Action):

    def __call__(self, parser, namespace, values, option_string=None):
        with open(values, 'r') as f:
            setattr(namespace, self.dest, f.read().splitlines())


target_file_oto = _TargetFileOTO
output_file_oto = _OutputFileOTO
target_file_otm = _TargetFileOTM
output_file_otm = _OutputFileOTM
watermark_mto = _WatermarkMTO
watermark_text = _WatermarkText



from io import BytesIO
from math import atan
from math import degrees
from math import sqrt
from PyPDF4.pdf import PdfFileReader
from reportlab.pdfgen.canvas import Canvas

from reportlab.lib.colors import Color


# noinspection PyTypeChecker
class Watermark:

    def __init__(self, text, page_size='A4', font_name='Helvetica-Bold',
                 font_size='medium',
                 text_alignment='diagonal', color='black', position_x='center',
                 position_y='center'):
        # TODO: think...
        self._page_size = page_size
        self._font_name = font_name
        self._font_size = font_size
        self._text_alignment = text_alignment
        self._color = color
        self._text = text
        self._position_x = position_x
        self._position_y = position_y
        self._x, self._y = 0, 0
        self._init_canvas()
        self._update_style()
        self._update_canvas()

    def _init_canvas(self):
        self._buffer = BytesIO()
        self._canvas = Canvas(self._buffer)

    def _set_rotation(self):
        x, y = list(map(float, self._page_size))
        rotation = 0
        if self._text_alignment == 'diagonal':
            rotation = round(degrees(atan(y / x)))
        if self._text_alignment == 'vertical':
            rotation = 90
        self._rotation = rotation

    def _set_max_length(self):
        x, y = list(map(int, self._page_size))
        length = round(x * .8)
        if self._text_alignment == 'diagonal':
            length = round(sqrt(x**2 + y**2) * .8)
        if self._text_alignment == 'vertical':
            length = round(y * .8)
        self._max_length = length

    def _calculate_font_size(self):
        x, y = list(map(int, self._page_size))
        max_width = round(self._max_length * .5)
        max_font = x*.1
        if self._font_size == 'medium':
            max_width = round(self._max_length * .75)
            max_font = x*.2
        if self._font_size == 'large':
            max_width = round(self._max_length)
            max_font = x*.3
        size = 5

        while True:
            width = self._canvas.stringWidth(self._text,
                                             fontName=self._font_name,
                                             fontSize=size)
            if width >= max_width or size >= max_font:
                break
            size += 5
        self._font_size = size
        self._width = width

    def _set_font_size(self):
        self._calculate_font_size()

    def _set_color(self):
        self._color = Color()

    def _set_position(self):
        # set to origin
        self._x = 0
        self._y = 0

    def _update_canvas(self):
        # update canvas attrs
        self._canvas.setPageSize(self._page_size)
        self._canvas.setFont(self._font_name, self._font_size)
        self._canvas.setFillColor(self._color)
        x, y = list(map(int, self._page_size))
        # set origin to center of the page
        self._canvas.translate(x // 2, y // 2)
        self._canvas.rotate(self._rotation)
        self._canvas.drawCentredString(self._x, -.25 * self._font_size,
                                       self._text)
        self._canvas.save()

    def _update_style(self):
        self._set_max_length()
        self._set_rotation()
        self._set_font_size()
        self._set_color()
        self._set_position()

    def unload(self):
        watermarked_pdf = PdfFileReader(self._buffer)
        return watermarked_pdf.getPage(0)



from pdf_play.core._pdf import PDF

# __all__ = ['PDF']
client = PDF(debug=True)


def watermark(text, target_file, output_file, **style):
    client.apply_watermark(text, target_file, output_file, **style)
    return True


__all__ = ['watermark']



from os.path import join
from pathlib import Path

# project dirs & files
BASE = Path(__file__).parent.resolve()
DATA = join(BASE, '.data')
FONTS = ['Courier', 'Courier-Bold', 'Courier-BoldOblique', 'Courier-Oblique', 'Helvetica',
         'Helvetica-Bold', 'Helvetica-BoldOblique', 'Helvetica-Oblique',
         'Times-Bold', 'Times-BoldItalic', 'Times-Italic', 'Times-Roman']



from PyPDF4.pdf import PdfFileReader
from PyPDF4.pdf import PdfFileWriter

from pdf_play.core._watermark import Watermark


class PDF:

    def __init__(self, debug=False):
        self._in_path = None
        self._target = None
        self._out_path = None
        self._out = None
        self._debug = debug
        self._watermark_text = ''
        self._watermark_is_loaded = False
        self._reader = PdfFileReader
        self._writer = PdfFileWriter

    def _reset(self, **kwargs):
        for key, value in kwargs.items():
            if value is None:
                raise AttributeError(f'{key} can not be None')
            setattr(self, key, value)

    def _set_in_path(self, path):
        assert isinstance(path, str), f'File path must be a string, received: {path}'
        if path and path != self._in_path:
            self._in_path = path
            self._target = PdfFileReader(self._in_path)

    def _set_out_path(self, path=None):
        if path is None:
            if self._in_path is not None:
                split = self._in_path.split('.')
                name, extension = '.'.join(split[:-1]), split[-1]
                path = f'{name}_watermarked.{extension}'
        else:
            if not isinstance(path, str):
                raise TypeError(f'Output path must be a string, received: {path}')
            if not path.endswith('.pdf'):
                raise TypeError(f'Output file must be a PDF, received: {path}')
        self._out_path = path
        self._out = PdfFileWriter()

    def _set_watermark_text(self, text=''):
        self._watermark_text = text

    def _set_watermark(self, **style):
        style['page_size'] = self._target.getPage(0).mediaBox[-2:]
        watermark = Watermark(self._watermark_text, **style)
        self._watermark = watermark.unload()
        self._watermark_is_loaded = True

    def _load_watermark(self, text, target_file, out_file, **style):
        try:
            self._set_watermark_text(text)
            self._set_in_path(target_file)
            self._set_out_path(out_file)
            self._set_watermark(**style)
        except Exception as e:
            print(f'Error: {e}')
            if self._debug:
                raise e

    def apply_watermark(self, text, target_file, out_file=None, **style):
        self._load_watermark(text, target_file, out_file, **style)
        if self._watermark_is_loaded:
            for i in range(self._target.getNumPages()):
                page = self._target.getPage(i)
                page.mergePage(self._watermark)
                self._out.addPage(page)

            with open(self._out_path, 'wb') as f:
                self._out.write(f)
            self._target.stream.close()


def _test():
    pdf = PDF()
    path = ''
    pdf.apply_watermark('test watermark!', path)


if __name__ == '__main__':
    _test()
